/**
* Klasse für einen (evtl. gebrochenen) Lichtstrahl
*/
function Strahl(
    x1,
    y1,
    x2,
    y2
) {
    x1 -= Strahl.LINSE_X;
    x2 -= Strahl.LINSE_X;
    y1 -= Strahl.LINSE_Y;
    y2 -= Strahl.LINSE_Y;
    if (x1)
        this.calc(x1, y1, x2 - x1, y2 - y1);
    else
        this.calc(x2, y2, x1 - x2, y1 - y2);
    this.draw();
}

// x-Koordinate der Linse
Strahl.LINSE_X = 50;
// y-Koordinate der optischen Achse
Strahl.LINSE_Y = 30;
// Brennweite
Strahl.BRENNWEITE = 40;
// Halbe Breite des Bildes
Strahl.HALFWIDTH = 52;
// Halbe Höhe des Bildes
Strahl.HALFHEIGHT = 32;

// Ankerpunkt
Strahl.prototype.x_vor = 0;
Strahl.prototype.y_vor = 0;

// Wird Linse getroffen?
Strahl.prototype.trifftLinse = undefined;

// y-Koordinate, wo Linse getroffen wird
Strahl.prototype.y_linse = undefined;

// Strahl nach Linse
Strahl.prototype.x_nach = 0;
Strahl.prototype.y_nach = 0;

/**
* Verlauf des Strahls berechnen
* @return undefined
*/
Strahl.prototype.calc = function(
    x_vor,
    y_vor,
    dx,
    dy
) {
    this.x_vor = x_vor;
    this.y_vor = y_vor;
    this.y_linse = y_vor - dy / dx * x_vor;
    this.trifftLinse = this.x_vor && (Math.abs(this.y_linse) <= Strahl.LINSE_Y);
    if (!this.trifftLinse) {
        while ((Math.abs(this.x_vor) < Strahl.HALFWIDTH) && (Math.abs(this.y_vor) < Strahl.HALFHEIGHT)) {
            this.x_vor += dx;
            this.y_vor += dy;
        }
        this.x_nach = x_vor;
        this.y_nach = y_vor;
        while ((Math.abs(this.x_nach) < Strahl.HALFWIDTH) && (Math.abs(this.y_nach) < Strahl.HALFHEIGHT)) {
            this.x_nach -= dx;
            this.y_nach -= dy;
        }
        return;
    }

    var p = this._verlaengere(x_vor, y_vor);
    this.x_vor = p.x;
    this.y_vor = p.y;
    var x_nach = 1 / (1 / Strahl.BRENNWEITE - 1 / Math.abs(this.x_vor));
    var virtual = (x_nach < 0);
    if (this.x_vor > 0)
        x_nach *= -1;
    var y_nach = x_nach * this.y_vor / this.x_vor;
    if (virtual) {
        x_nach *= -1;
        y_nach = this.y_linse - y_nach;
    }

    p = this._verlaengere(x_nach, y_nach);
    this.x_nach = p.x;
    this.y_nach = p.y;
}

Strahl._verlaengere = function(
    x0,
    y0,
    x,
    y
) {
    if (!Number.isFinite(y))
        return { x: 0, y: -Strahl.HALFHEIGHT };
    if (!Number.isFinite(y0))
        return { x: 0, y: Strahl.HALFHEIGHT };
    var dx = x - x0;
    var dy = y - y0;
    var n_min = [(Strahl.HALFWIDTH - x0) / dx, (-Strahl.HALFWIDTH - x0) / dx, (Strahl.HALFHEIGHT - y0) / dy, (-Strahl.HALFHEIGHT - y0) / dy].filter(function(v) {return (v > 0)}).reduce(function(a, b) { return Math.min(a, b); });
    return { x: x0 + n_min * dx, y: y0 + n_min * dy };
}

/**
* Verlängert den Strahl von (0|this.y_linse) durch (x|y) bis garantiert außerhalb des Bildes
* @return Object    { x: "x-Koordinate des Punktes garantiert außerhalb", y: "y-Koordinate des Punktes garantiert außerhalb" }
*/
Strahl.prototype._verlaengere = function(
    x,          // x-Koordinate des Punktes, durch den der Strahl gehen soll
    y           // y-Koordinate des Punktes, durch den der Strahl gehen soll
) {
    return Strahl._verlaengere(0, this.y_linse, x, y);
}

/**
* Verlauf des Strahls zeichnen
* @return undefined
*/
Strahl.prototype.draw = function() {
    var d = 'M' + (this.x_vor + Strahl.HALFWIDTH) + ',' + (this.y_vor + Strahl.HALFHEIGHT) + 'L';
    if (this.trifftLinse)
        d += Strahl.HALFWIDTH + ',' + (this.y_linse + Strahl.HALFHEIGHT) + ' ';
    d += (this.x_nach + Strahl.HALFWIDTH) + ',' + (this.y_nach + Strahl.HALFHEIGHT);
    this.node = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    this.node.setAttribute('d', d);
    document.getElementById('layer1').appendChild(this.node);
}

/**
* Strahl aus SVG entfernen
* @return undefined
*/
Strahl.prototype.remove = function() {
    if (this.node.parentNode)
        this.node.parentNode.removeChild(this.node);
}
