/**
* Klasse für einfache Aufgaben, bei denen es ein auditives Feedback gibt
*/
function AudioFeedbackTask() {}

/**
* HTML-Element in DOM-Baum einfügen
* @return Element   Erzeugter Knoten
*/
AudioFeedbackTask.createHtml = function(
    strTagname,     // Optional: Tagname des zu erzeugenden Elementes
    nodeParent,     // Optional: Künftiger Elternknoten
    objAtts,        // Optional: Attribute des zu erzeugenden Elementes
    strTextContent, // Optional: Textinhalt des Elementes
    nodeBefore      // Optional: Knoten, als deren previousSibling der neue Knoten eingefügt werden soll
) {
    var node = nodeBefore ? nodeBefore.parentNode : nodeParent;
    if (nodeBefore && nodeBefore.parentNode)
        nodeParent = nodeBefore.parentNode;
    if (strTagname)
        node = (nodeParent ? nodeParent.ownerDocument : document).createElementNS('htt' + 'p://w' + 'ww.w3.org/1999/xhtml', strTagname);
    if (objAtts)
        for (var key in objAtts)
            node.setAttribute(key, objAtts[key]);
    if ((strTextContent !== undefined) && ('' + strTextContent).length)
        node.appendChild((nodeParent ? nodeParent.ownerDocument : document).createTextNode(strTextContent));
    if (nodeParent && (node != nodeParent))
        nodeParent.insertBefore(node, nodeBefore);
    return node;
}

/**
* Schaltflächen zum Ein-/Ausschalten der Audioausgabe und Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
AudioFeedbackTask.prototype.establish = function() {
    // Schaltfläche für Lizenzinfos
    (function(node) {
        if (!node || !node.firstChild)
            return;
        var nodeA = node.firstChild;
        nodeA.addEventListener('click', function(ev) {
            ev.preventDefault();
            ev.stopPropagation();
            var node = document.createElement('div');
            node.className = 'modal';
            node.addEventListener('click', function() { node.parentNode.removeChild(node); });
            node.appendChild(document.createElement('div'));
            node.firstChild.appendChild(document.createElement('div'));
            var iframe = document.createElement('iframe');
            iframe.style.height = '33vh';
            iframe.addEventListener('load', function() {
                try {
                    iframe.style.height = Math.min((iframe.contentDocument ? iframe.contentDocument : iframe.contentWindow.document).body.parentNode.scrollHeight + 10, window.innerHeight - 32) + 'px';
                } catch {}
            });
            node.firstChild.firstChild.appendChild(iframe);
            var close = document.createElement('a');
            close.href='#';
            close.addEventListener('click', function(ev) { ev.preventDefault(); ev.stopPropagation(); node.parentNode.removeChild(node); });
            close.appendChild(document.createTextNode('Infokasten schließen'));
            node.firstChild.firstChild.appendChild(document.createElement('p'));
            node.firstChild.firstChild.lastChild.appendChild(close);
            document.body.appendChild(node);
            iframe.src = nodeA.href;
            close.focus();
        });
    })(document.getElementsByTagName('footer')[0]);
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
AudioFeedbackTask.prototype.connect = function() {
    if (typeof LMSLink == 'function') {
        this.lmslink = new LMSLink();
        this.lmslink.start();
        if (this.lmslink && this.lmslink.ready) {
            if (this.lmslink.preferenceAudio == -1)
                this.muted = true;
        }
    }
    if (!this.lmslink || !this.lmslink.ready)
        this.showMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn', true);
    if (this.getDone()) {
        if (this.lmslink.lessonStatus == 'failed')
            this.showMsg('Aufgabe wurde engültig nicht geschafft', 'msg_failed', true);
        else
            this.showMsg('Aufgabe wurde bereits früher erfolgreich erledigt', 'msg_success', true);
    }
}

// Genutzte Instanz von LMSLink
AudioFeedbackTask.prototype.lmslink = null;

// Wurde das auditive Feedback stumm geschaltet?
AudioFeedbackTask.prototype.muted = false;

// Abzuspielendes Audio, wenn richtige Lösung zusammengestellt wurde
AudioFeedbackTask.audioSuccess = null;

// Abzuspielendes Audio, wenn eine Lösung falsch ist
AudioFeedbackTask.audioFail = null;

// Zu cachende bzw. später gecachte Grafiken
AudioFeedbackTask.caching = ['not-ok.svg', 'ok.svg'];

/**
* Lädt Audio- und Bilddateien
* @return undefined
*/
AudioFeedbackTask.cache = function(
    boolLoadAudio       // Auch Audiodateien laden?
) {
    if (!AudioFeedbackTask.audioFail && boolLoadAudio)
        AudioFeedbackTask.audioFail = new Audio('share/fail.mp3');
    if (!AudioFeedbackTask.audioSuccess && boolLoadAudio)
        AudioFeedbackTask.audioSuccess = new Audio('share/success.mp3');
    AudioFeedbackTask.caching = AudioFeedbackTask.caching.map(function(filename) {
        if ((typeof filename == 'string') || (filename instanceof String)) {
            var img = new Image();
            img.src = 'share/' + filename;
            return { "filename": filename, "img": img };
        }
        return filename;
    });
}

// Aktive Meldung
AudioFeedbackTask.prototype.nodeMsg = null;

// DOM-Element, vor dem Meldungen eingefügt werden sollen
AudioFeedbackTask.prototype.nodeBeforeMsg = null;

// DOM-Element, vor dem Meldungen eingefügt werden sollen, die generell sind
AudioFeedbackTask.prototype.nodeBeforeGeneralMsg = null;

/**
* Eine Meldung anzeigen
* @return undefined
*/
AudioFeedbackTask.prototype.showMsg = function(
    strText,        // Textinhalt der Meldung
    strClass,       // Klassenname der Meldung
    boolGeneral     // Optional: Ist die Meldung generell und permanent?
) {
    var node = AudioFeedbackTask.createHtml('p', null, { "class": strClass });
    AudioFeedbackTask.createHtml('span', AudioFeedbackTask.createHtml('span', node), null, strText);
    if (boolGeneral) {
        var nodeBefore = this.nodeBeforeGeneralMsg ? this.nodeBeforeGeneralMsg : this.nodeBeforeMsg;
        nodeBefore.parentNode.insertBefore(node, nodeBefore);
    } else {
        if (this.nodeMsg)
            this.nodeMsg.parentNode.replaceChild(node, this.nodeMsg);
        else
            this.nodeBeforeMsg.parentNode.insertBefore(node, this.nodeBeforeMsg);
        this.nodeMsg = node;
    }
}

/**
* Zuletzt angezeigte Meldung löschen
* @return undefined
*/
AudioFeedbackTask.prototype.clearMsg = function() {
    if (this.nodeMsg && this.nodeMsg.parentNode) {
        this.nodeMsg.parentNode.removeChild(this.nodeMsg);
        this.nodeMsg = null;
    }
}

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
AudioFeedbackTask.prototype.getDone = function() {
    return (this.lmslink && this.lmslink.ready && (['completed', 'passed', 'failed'].indexOf(this.lmslink.lessonStatus) != -1));
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
AudioFeedbackTask.prototype.canSave = function() {
    if (this._canSave === undefined)
        this._canSave = (this.lmslink && this.lmslink.ready && (this.lmslink.lessonMode == 'normal'));
    return (this._canSave && (['completed', 'passed', 'failed'].indexOf(this.lmslink.lessonStatus) == -1));
}

/**
* Den Zwischenstand an Bearbeitungszeit an die Lernplattform melden
* @return undefined
*/
AudioFeedbackTask.prototype.reportTime = function() {
    if (this.lmslink && this.lmslink.ready)
        this.lmslink.sessionTime += 0;
}

/**
* Eine auditive Rückmeldung abspielen (sofern diese verfügbar und erwünscht ist)
* @return undefined
*/
AudioFeedbackTask.prototype.playAudio = function(
    boolSuccess     // Wurde ein Erfolg erzielt?
) {
    if (this.muted)
        return;
    var audio = boolSuccess ? AudioFeedbackTask.audioSuccess : AudioFeedbackTask.audioFail;
    if (audio && audio.play)
        audio.play();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
AudioFeedbackTask.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (this.canSave()) {
        this.lmslink.lessonStatus = status ? status : 'incomplete';
        this.lmslink.scoreMin = scoreMin;
        this.lmslink.scoreMax = scoreMax;
        this.lmslink.score = scoreRaw;
    }
    this.reportTime();
}
